/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/

#include "BLEHandler.h"

#include "debug.h"


CYBLE_CONN_HANDLE_T cyBle_connHandle; /* BLE connection handle */

static  uint8 connectionIrk[CYBLE_GAP_SMP_IRK_SIZE]; /* IRK information of a bonded device */

static  ADVERTISEMENT_STATE advState; /* state of the advertisment */

static  AUTHENTICATION_STATE authState; /* state use to know if there is a bonded device */

static uint32 u32Passkey = (123456u); /* Fixed password for Bluetooth connection */

static uint8 u8ConnectionFlag = 0; /* connection is active */

static uint8 u8DeleteBondingFlag = 1; /* flag to signal that the bonding information can be deleted */

/************************ private functions  declaration ************************************/

/************************ private functions definition **************************************/
static void AdvertisePublicly(void)
{
    /* Set the original public address as the device address */
    CyBle_SetDeviceAddress(&cyBle_deviceAddress);
    
    /* Revert if any changes made to advertisement settings */
    cyBle_discoveryModeInfo.advData = &cyBle_discoveryData;
    cyBle_discoveryParam.ownAddrType = CYBLE_GAP_ADDR_TYPE_PUBLIC;
    CyBle_GappStartAdvertisement(CYBLE_ADVERTISING_FAST);
}

static void AdvertisePrivately(void)
{
    
    CYBLE_GAP_BD_ADDR_T privateAddress; /* private address to be generated */
    /* Create a new data structure for Advertisement data without the device name.*/
    CYBLE_GAPP_DISC_DATA_T privateAdvData = 
    {
        {
            0x02,
            0x01,
            0x06
        },
        3
    };
    
    /* Generate a new resolvable private address */
    if(CYBLE_ERROR_OK == CyBle_GapGenerateDeviceAddress(&privateAddress, CYBLE_GAP_RANDOM_PRIV_RESOLVABLE_ADDR, connectionIrk))
    {
        /* Set the new device address */
        while(CyBle_SetDeviceAddress(&privateAddress) != CYBLE_ERROR_OK);
        
        /* Advertise but without the device name */
        cyBle_discoveryModeInfo.advData = &privateAdvData;
        cyBle_discoveryParam.ownAddrType = CYBLE_GAP_ADDR_TYPE_RANDOM;
        CyBle_GappStartAdvertisement(CYBLE_ADVERTISING_FAST);
        
        DBG_PRINT_TEXT("Advertising with private address: ");
        for(uint8 u8I = 6; u8I > 0; u8I--)
        {
            DBG_PRINT_HEX_TEXT(privateAddress.bdAddr[u8I - 1], " ");
        }
        DBG_PRINT_TEXT("\n");
    }
}


inline static void BLE__vClockConfigInDisconnectState(void)
{
    CYBLE_BLESS_CLK_CFG_PARAMS_T clockConfig;
 
    /* get the configrated parameter */    
    (void)CyBle_GetBleClockCfgParam(&clockConfig); 
    /*set the device sleep clock acuracy based on the tuned ppm of the WCO */
    clockConfig.bleLlSca = CYBLE_LL_SCA_000_TO_020_PPM;
    /*set clock parameters*/
    (void)CyBle_SetBleClockCfgParam(&clockConfig);
    /* no connection */
    u8ConnectionFlag = 0; 
    /* Turn off the LED */
    LED_Write(1); 
}

/* Callbacks handler for BLE */
static void BLEStackCallback(uint32 event, void* eventParam)
{
    CYBLE_GAP_SMP_KEY_DIST_T * keys;
    
    /* common events for advertisment with public or private addess */
    switch(event)
    {
        case CYBLE_EVT_GATT_CONNECT_IND:
            DBG_PRINT_TEXT("connection indication\n");
            u8ConnectionFlag = 1;
            cyBle_connHandle = *(CYBLE_CONN_HANDLE_T *) eventParam;
            LED_Write(0); /* Turn on the LED */
            break;
         
        case CYBLE_EVT_GAP_AUTH_REQ: 
            DBG_PRINT_TEXT("Authentication request.\n");
              //Request to Initiate Authentication received
            CyBle_GappAuthReqReply(cyBle_connHandle.bdHandle,&cyBle_authInfo);
            break;
            
         case CYBLE_EVT_GAP_KEYINFO_EXCHNGE_CMPLT:
            DBG_PRINT_TEXT("Exchnage info.\n");
            /* Copy the IRK key to a buffer; to be used for creating a private address.*/
            keys = (CYBLE_GAP_SMP_KEY_DIST_T *) eventParam;
            memcpy(connectionIrk, keys->irkInfo, CYBLE_GAP_SMP_IRK_SIZE);
            break;    
            
        case CYBLE_EVT_GAP_AUTH_COMPLETE:
            DBG_PRINT_TEXT("Authentication complete.\n");
            authState = AUTHENTICATION_COMPLETE_BONDING_REQD;
            break;
        
        case CYBLE_EVT_GAP_AUTH_FAILED:
            DBG_PRINT_TEXT("Authentication failed.\n");
            (void)CyBle_GapDisconnect(cyBle_connHandle.bdHandle);
            /* Update authentication state so that bond information could be removed later */
            authState = AUTHENTICATION_BONDING_COMPLETE;
            break;    
                        
        default:
            /* should not be here */
        break;
    }
    
    switch(advState)
    {
         /* If the bonding is not done and we are trying to connect to any 
         * device for the first time.
         */
        case ADVERTISEMENT_PUBLIC:
            switch(event)
            {
                case CYBLE_EVT_STACK_ON:
                     /* Stack initialized; start advertisement */
                    DBG_PRINT_TEXT("Advertising with public address: ");
                    for(uint8 u8I = 6; u8I > 0; u8I--)
                    {
                        DBG_PRINT_HEX_TEXT(cyBle_deviceAddress.bdAddr[u8I - 1], " ");
                    }
                    DBG_PRINT_TEXT("\n");
                
                    /*Set the passkey to "123456"*/
                    (void)CyBle_GapFixAuthPassKey(FIX_PASSKEY, u32Passkey);
                    /* Intentional Fallthrough */
                    AdvertisePublicly();
                    break;
                    
                case CYBLE_EVT_GAPP_ADVERTISEMENT_START_STOP:
                    /* If advertisement timed out and we were not connected 
                     * yet, restart advertisement.
                     */
                    if(authState != AUTHENTICATION_BONDING_REMOVE_WAITING_EVENT)
                    {
                        if(CyBle_GetState() == CYBLE_STATE_DISCONNECTED)
                        {
                            AdvertisePublicly();
                        }
                    }
                    else
                    {
                        DBG_PRINT_TEXT("Advertisement stopped. \n");
                        authState = AUTHENTICATION_BONDING_REMOVE_GO_AHEAD;
                    }
                    break;

                case CYBLE_EVT_GAP_DEVICE_DISCONNECTED:                
                    BLE__vClockConfigInDisconnectState();
                    /* Disconnected but bonding is not complete; restart 
                     * advertisement.
                     */
                    DBG_PRINT_TEXT("\n\n\rDisconnected. ");
                    if(authState != AUTHENTICATION_BONDING_REMOVE_WAITING_EVENT)
                    {
                        DBG_PRINT_TEXT("Advertising with public address: ");
                        for(uint8 u8I = 6; u8I > 0; u8I--)
                        {
                            DBG_PRINT_HEX_TEXT(cyBle_deviceAddress.bdAddr[u8I - 1], " ");
                        }
                        AdvertisePublicly();
                    }
                    else
                    {
                        authState = AUTHENTICATION_BONDING_REMOVE_GO_AHEAD;
                    }
                    break;
                
                default:
                    break;
            }
            
        break;
            
        /* If the bonding has already happened and we are advertising with 
         * a private address.
         */        
        case ADVERTISEMENT_PRIVATE:
            switch(event)
            {
                case CYBLE_EVT_STACK_ON:
                    /* Stack initialized; start advertisement */
                    AdvertisePrivately();
                    break;
                    
                case CYBLE_EVT_GAPP_ADVERTISEMENT_START_STOP:
                    BLE__vClockConfigInDisconnectState();
                    /* If advertisement timed out and we were not connected 
                     * yet, restart advertisement.
                     */
                    if(authState != AUTHENTICATION_BONDING_REMOVE_WAITING_EVENT)
                    {
                        if(CyBle_GetState() == CYBLE_STATE_DISCONNECTED)
                        {
                            DBG_PRINT_TEXT("160 seconds elapsed. \n");
                            AdvertisePrivately();
                        }
                    }
                    else
                    {
                        DBG_PRINT_TEXT("Advertisement stopped. ");
                        authState = AUTHENTICATION_BONDING_REMOVE_GO_AHEAD;
                    }
                    break;
                
                case CYBLE_EVT_GAP_DEVICE_DISCONNECTED:
                    /* Disconnected and bonding was previously complete; start 
                     * advertisement with private resolvable address but only 
                     * if the bonding is not to be removed. If the bonding is 
                     * to be removed then advertisement with public address
                     * will be started after the bonding is removed.
                     */
                    DBG_PRINT_TEXT("Disconnected. \n");
                    if(authState != AUTHENTICATION_BONDING_REMOVE_WAITING_EVENT)
                    {
                        AdvertisePrivately();
                    }
                    else
                    {
                        authState = AUTHENTICATION_BONDING_REMOVE_GO_AHEAD;
                    }
                    break;
                
                default:
                    break;
            }
        break;
            
        default:
            
            break;
    }
}

/************************ Public functions definition **************************************/
void BLEHandler_vStart(void)
{
//    CYBLE_GAP_BONDED_DEV_ADDR_LIST_T stBondedDeviceList; /* list of bounded devices */
//    CYBLE_GAP_BD_ADDR_T stBondedDeviceAddress;
//    uint8 u8BondedDeviceBdHandle;
//    uint8 u8BondedDeviceKeysFlag;
    
    /* Start the BLE with the call back associated */
    CyBle_Start(BLEStackCallback);
    
    /* Find out whether the device has bonded information stored already or not */
  //  CyBle_GapGetBondedDevicesList(&stBondedDeviceList);
    //TODO uncoment this at reset and remove bond at button press, temporary let it commented
//    if(stBondedDeviceList.count != 0)
//    {
//        CYBLE_GAP_SMP_KEY_DIST_T securityKeys;
//        
//        /* Take the last bonded device */
//        memcpy(stBondedDeviceAddress.bdAddr, stBondedDeviceList.bdAddrList[0].bdAddr, 6);
//        stBondedDeviceAddress.type = stBondedDeviceList.bdAddrList[0].type;
//
//        /* Get the BD handle of that bonded device */
//        CyBle_GapGetPeerBdHandle(&u8BondedDeviceBdHandle, &stBondedDeviceAddress);
//        
//        /* Get the IRK for that bonded device */
//        u8BondedDeviceKeysFlag |= 0x01;   /* Identity Resolution Key */ 
//        CyBle_GapGetPeerDevSecurityKeyInfo(u8BondedDeviceBdHandle, &u8BondedDeviceKeysFlag, &securityKeys);
//        memcpy(connectionIrk, securityKeys.irkInfo, CYBLE_GAP_SMP_IRK_SIZE);
//        
//        /* Start advertisement with private address */
//        advState = ADVERTISEMENT_PRIVATE;
//        authState = AUTHENTICATION_BONDING_COMPLETE;
//    }
//    else
    {
        /* Start advertisement with public address */
        advState = ADVERTISEMENT_PUBLIC;
        authState = AUTHENTICATION_BONDING_REMOVE_GO_AHEAD;//AUTHENTICATION_NOT_CONNECTED;
    }
}

/* this should be called every time something was done with the BLE */
void BLEHandler_vPrivacyTask(void)
{
    /* All zeros passed as  argument passed to CyBle_GapRemoveDeviceFromWhiteList for removing all the bonding data stored */
    CYBLE_GAP_BD_ADDR_T clearAllDevices = {{0,0,0,0,0,0},0};
    
    switch(authState)
    {            
        case AUTHENTICATION_COMPLETE_BONDING_REQD:
            /* Store bonding data of the current connection */
            while(CYBLE_ERROR_OK != CyBle_StoreBondingData(0));
            
            authState = AUTHENTICATION_BONDING_COMPLETE;
            DBG_PRINT_TEXT("Bonding complete Requested\n");
            
            /* Ready for private advertisement the next time */
            advState = ADVERTISEMENT_PRIVATE;
            break;
            
            
        case AUTHENTICATION_BONDING_COMPLETE:
            /* if the user pres the reset/bond button when device is on, remove bonding */
            if(!u8DeleteBondingFlag)
            {
                if(CyBle_GetState() == CYBLE_STATE_CONNECTED)
                {
                    /* Disconnect */
                    authState = AUTHENTICATION_BONDING_REMOVE_WAITING_EVENT;
                    CyBle_GapDisconnect(cyBle_connHandle.bdHandle);
                    DBG_PRINT_TEXT("Bonding complete remove bonded connected.\n");
                }
                else if(CyBle_GetState() == CYBLE_STATE_ADVERTISING)
                {
                    /* Stop advertisement */
                    authState = AUTHENTICATION_BONDING_REMOVE_WAITING_EVENT;
                    CyBle_GappStopAdvertisement();
                    DBG_PRINT_TEXT("Bonding complete remove bonding advertising.\n");
                }
                else
                {
                    authState = AUTHENTICATION_BONDING_REMOVE_GO_AHEAD;
                    DBG_PRINT_TEXT("Bonding complete remove bonding go.\n");
                }
            }
            break;

            
        case AUTHENTICATION_BONDING_REMOVE_GO_AHEAD:
            /* Remove all bonded devices in the list */
            CyBle_GapRemoveDeviceFromWhiteList(&clearAllDevices); 
            while(CYBLE_ERROR_OK != CyBle_StoreBondingData(1));
            DBG_PRINT_TEXT("Cleared the list of bonded devices. \n");
            
            /* Start advertising with a public address */
            advState = ADVERTISEMENT_PUBLIC;
            DBG_PRINT_TEXT("Advertising with public address: ");
            for(uint8 u8I = 6; u8I > 0; u8I--)
            {
                DBG_PRINT_HEX_TEXT(cyBle_deviceAddress.bdAddr[u8I - 1], " ");
            }
            AdvertisePublicly();              
            authState = AUTHENTICATION_NOT_CONNECTED;
            u8DeleteBondingFlag = 1;
            break;

        default:
            break;
    }
}

void BLEHandler_vSetRemoveBondedData(uint8 u8Flag){u8DeleteBondingFlag = u8Flag;}
/* [] END OF FILE */
